# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateClusterViaR(Template):       
    'Performs a cluster analysis using either the <a href="http://stat.ethz.ch/R-manual/R-patched/library/stats/html/00Index.html" target="blank">stats</a> package or <a href="http://cran.r-project.org/web/packages/cluster/cluster.pdf" target="blank">cluster</a> package in R. '

    __version__ = '1.0.0'

    tags = ['Cluster analysis' ]
    engines = ['R_script']

    inputs = '''
x = DataMatrix('random variables: ', allow_cat = True)
stand = Boolean('standardize the data? ')
if stand:
    center = Boolean('substract the means? ')
    scale  = Boolean('scale each variable? ')
distMethod =  Text('specify the distance measure: ', ['euclidean', 'squared_euclidean','maximum', 'manhattan', 'canberra', 'binary', 'minkowski'])
if distMethod == 'minkowski':
    p = DataVector('the power of the Minkowski distance: ')
Alg = Text('specify clustering algorithm: ', ['Hierarchical Clustering','K-Means Clustering', 'Partitioning Around Medoids','Clustering Large Applications'])

if Alg =='Hierarchical Clustering':
    method = Text('specify the agglomeration method: ', [ 'ward', 'single', 'complete', 'average', 'mcquitty', 'median', 'centroid'])
    if distMethod=='squared_euclidean' and method=='centroid':
        nok =  Boolean('specify the number of clusters? ')
        if nok:
            k = Text('specify the value: ')

    anylabel = Boolean('use labels? ')
    if anylabel:
        defaultlabel =  Boolean('use the row numbers as labels? ')
        if not defaultlabel:
            labels = DataVector('specify labels: ')
    
if Alg =='Partitioning Around Medoids':
    k = Text('specify number of clusters: ')
    init = Boolean('use the build algorithm to obtain the initial values of medoids? ')
    if not init:
        medoids = Text('specify medoids: ')


if Alg=='K-Means Clustering':
    centers = Text('specify either the number of clusters or a set of initial (distinct) cluster centrers: ')
    iterMax = Text('specify the maximum number of iterations: ')
    if len(centers.name)==1:
        nstart = Text('specify the number of random sets: ')
    algorithm = Text('choose an algorithm: ', ["Hartigan-Wong", "Lloyd", "Forgy","MacQueen"])
    cplot = Boolean('produce cluster plot (using the cluster package)? ')

if Alg == 'Clustering Large Applications':
    k = Text('specify number of clusters: ')
    metric=Text('specify the distance measure: ', ['euclidean',  "manhattan"])
    samples=Text('number of samples to be drawn from the dataset: ')
    samplesizedef=Boolean('use default sample size, i.e., min(n, 40 + 2 * k): ')
    if not samplesizedef:
        samplesize=Text('specify the sample size: ')
    reliability =Boolean('Check reliability? ')
    if reliability:
        nsim=Text('the number of simultions: ')
'''

    rscript = '''
X =NULL
%for i in range(len(x)):
tt=mydata$${x[i]}
tt[tt==Inf]=NA
X=cbind(X, tt)
%endfor
X=as.matrix(X)
%for i in range(len(x)):
colnames(X)[${i+1}]='${x.name[i]}'
%endfor

%if stand:
%if center:
%if scale:
X=scale(X,center=T,scale=T)
%else:
X=scale(X,center=T,scale=F)
%endif
%else:
%if scale:
#dividing by the root mean square for all r.v.
X=scale(X,center=F,scale=T)
%endif
%endif
%endif

%if Alg !='Clustering Large Applications':
%if distMethod =='squared_euclidean':
dm=dist(X, method='euclidean')^2
%else:
%if distMethod == 'minkowski':
dm= dist(X, method ='${distMethod}', p=${p})
%else:
dm= dist(X, method ='${distMethod}')
%endif
%endif
%endif

%if Alg =='Hierarchical Clustering':
hc=hclust(dm, method = '${method}')


write.dta(data.frame(hc$merge),file="merge.dta")
write.dta(data.frame(hc$order),file="order.dta")
write.dta(data.frame(hc$height),file="height.dta")

svg('hclustPlot.svg')
%if anylabel:
%if not defaultlabel:
plot(hc,labels=mydata$${labels})
%else:
plot(hc)
%endif
%else:
plot(hc,labels=FALSE)
%endif
dev.off()

%if nok and str(method)=="centroid" and str(distMethod)=="squared_euclidean":
memb <- cutree(hc, k=${int(k)})
cent <- NULL
for(k in 1:${int(k)}){
  cent <- rbind(cent, colMeans(X[memb == k, , drop = FALSE]))
}

hc1 <- hclust(dist(cent, method ='euclidean')^2, method = "cen", members = table(memb))


write.dta(data.frame(hc1$merge),file="merge1.dta")
write.dta(data.frame(hc1$order),file="order1.dta")
write.dta(data.frame(memb), file="members1.dta")
write.dta(data.frame(hc1$height),file="height1.dta")

svg('hclustPlot1.svg')
%if anylabel:
%if not defaultlabel:
plot(hc1,labels=mydata$${labels})
%else:
plot(hc1)
%endif
%else:
plot(hc1,labels=FALSE)
%endif
dev.off()
%endif
%endif

%if Alg=='Partitioning Around Medoids':
PACKages<-as.character(data.frame(installed.packages())$Package)
test<-("cluster" %in% PACKages)
if (!test){
	install.packages("cluster",repos="http://cran.r-project.org")
}
library(cluster)

%if init:
pamx = pam(dm, ${int(k)})
%else:
medoids=rep(0,${len(medoids.name)})
%for i in range(len(medoids.name)):
medoids[${i+1}]=${int(medoids.name[i])}
%endfor
pamx = pam(dm, ${int(k)},medoids=medoids)
%endif

summary(pamx)
svg('clusplot.svg')
clusplot(pamx)
dev.off()

svg('silhouette.svg')
plot(silhouette(pamx),col=1:${int(k)})
dev.off()

write.dta(data.frame(pamx$clustering),file="clustering.dta")
write.dta(data.frame(pamx$medoids), file="medoids.dta")
%endif

%if Alg=='K-Means Clustering':
%if cplot:
PACKages<-as.character(data.frame(installed.packages())$Package)
test<-("cluster" %in% PACKages)
if (!test){
	install.packages("cluster",repos="http://cran.r-project.org")
}
library(cluster)
%endif

%if len(centers.name)==1:
km=kmeans(X,centers=${int(centers)},iter.max=${int(iterMax)},nstart=${nstart},algorithm='${algorithm}')
%else:
centers=rep(0,${len(centers.name)})
%for i in range(len(centers.name)):
centers[${i+1}]=${centers.name[i]}
%endfor
km=kmeans(X,centers=centers,iter.max=${int(iterMax)},algorithm='${algorithm}')
%endif

km

write.dta(data.frame(km$cluster),file="cluster.dta")
write.dta(data.frame(km$centers),file="centers.dta")
%if cplot:
svg('clusplot.svg')
clusplot(x=dm,clus=km$cluster,diss=T)
dev.off()
%endif
%endif

%if Alg=='Clustering Large Applications':
PACKages<-as.character(data.frame(installed.packages())$Package)
test<-("cluster" %in% PACKages)
if (!test){
	install.packages("cluster",repos="http://cran.r-project.org")
}
library(cluster)

%if not samplesizedef:
clarax=clara(x=X, k=${int(k)},metric='${str(metric)}',samples=${int(samples)},sampsize=${int(samplesize)})
%else:
clarax=clara(x=X, k=${int(k)},metric='${str(metric)}',samples=${int(samples)})
%endif

summary(clarax)
clarax$clusinfo

svg('clusplot.svg')
clusplot(clarax)
dev.off()

svg('silhouette.svg')
plot(silhouette(clarax),col=1:${int(k)})
dev.off()

write.dta(data.frame(clarax$sample),file="bestSample.dta")
write.dta(data.frame(clarax$clustering),file="clustering.dta")
write.dta(data.frame(cbind(clarax$i.med,clarax$medoids)), file="medoids.dta")
write.dta(as.data.frame(as.matrix(clarax$diss)), file="dissimilarity.dta")

%if reliability:
nsim = ${int(nsim)}
cl = matrix(NA, length(mydata$${x[0]}),${int(nsim)})
set.seed(421)# (reproducibility)
for (i in 1:${int(nsim)}){
%if not samplesizedef:
cl[,i]=clara(x=X, k=${int(k)},metric='${str(metric)}',samples=${int(samples)},sampsize=${int(samplesize)}, medoids.x = FALSE, rngR = TRUE)$cluster
%else:
cl[,i]=clara(x=X, k=${int(k)},metric='${str(metric)}',samples=${int(samples)}, medoids.x = FALSE, rngR = TRUE)$cluster
%endif
}
tcl = apply(cl,1, tabulate, nbins = ${int(k)})
## those that are not always in same cluster:
(iDoubt <- which(apply(tcl,2, function(n) all(n < nsim))))
if(length(iDoubt)) { 
  tabD <- tcl[,iDoubt, drop=FALSE]
  dimnames(tabD) <- list(cluster = paste(1:${int(k)}), obs = format(iDoubt))
  t(tabD) # how many times in which clusters
  write.dta(data.frame(t(tabD)),file="tabDoubt.dta")
}

%endif
%endif
'''
